@@ -22,15 +22,36 @@
 #            Johan Dahlin <jdahlin@async.com.br>
 #
 
+"""Interface specifications and utilities"""
+
 import sys
 
 import gobject
 
 class Mixin(object):
-    pass
-
+    # gsignal('content-changed')
+    # gproperty('data-type')
+    # gproperty('model-attribute')
+    # gproperty('default-value')
+    # gproperty('validation-changed')
+    
+    def read(self):
+        pass
+    
+    def update(self, value):
+        pass
+    
 class MixinSupportValidation(object):
-    pass
+    # gproperty('mandatory')
+    def is_valid(self):
+        pass
+
+    # These two are mainly used by subclasses
+    def validate_data(self, data):
+        pass
+    
+    def before_validate(self, data):
+        pass
 
 class AbstractGladeAdaptor(object):
     """Abstract class that define the functionality an class that handle
