@@ -39,11 +39,13 @@ from kiwi import _warn
 from kiwi.environ import is_gazpacho_required
 from kiwi.interfaces import MixinSupportValidation
 from kiwi.proxies import Proxy
-from kiwi.utils import gsignal
+from kiwi.utils import gsignal, type_register
 from kiwi.ui.gadgets import quit_if_last
 
+WidgetTree = None
 try:
-    from kiwi.ui.gazpacholoader import GazpachoWidgetTree as WidgetTree
+    from gazpacho.loader import loader
+    loader # pyflakes
 except ImportError, e:
     if is_gazpacho_required():
         raise RuntimeError("Gazpacho is required, but could not be found: %s"
@@ -51,8 +53,11 @@ except ImportError, e:
     else:
         try:
             from kiwi.ui.libgladeloader import LibgladeWidgetTree as WidgetTree
+            WidgetTree # pyflakes
         except ImportError:
             raise RuntimeError("Could not find a glade parser library")
+else:
+    from kiwi.ui.gazpacholoader import GazpachoWidgetTree as WidgetTree
         
 _non_interactive = [
     gtk.Label, 
@@ -80,6 +85,52 @@ _non_interactive = [
 
 _non_interactive = tuple(_non_interactive)
 
+class signal_block:
+    """
+    A decorator to be used on view methods.
+    It takes a list of arguments which is the name of the widget and
+    the signal name separated by a dot.
+
+    For instance:
+
+        >>> class MyView(SlaveView):
+        ...     @block('money.changed')
+        ...     def update_money(self):
+        ...         self.money.set_value(10)
+        ...     def on_money__changed(self):
+        ...         pass
+
+
+    When calling update_money() the value of the spinbutton called money
+    will be updated, but on_money__changed will not be called.
+    """
+    
+    def __init__(self, *signals):
+        self.signals = []
+        for signal in signals:
+            if not isinstance(signal, str):
+                raise TypeError("signals must be a list of strings")
+            if signal.count('.') != 1:
+                raise TypeError("signal must have exactly one dot")
+            self.signals.append(signal.split('.'))
+
+    def __call__(self, func):
+        def wrapper(view, *args, **kwargs):
+            for name, signal in self.signals:
+                widget = getattr(view, name, None)
+                if widget is None:
+                    raise TypeError("Unknown widget %s in view " % name)
+                view.handler_block(widget, signal)
+                
+            retval = func(view, *args, **kwargs)
+            
+            for name, signal in self.signals:
+                widget = getattr(view, name, None)
+                view.handler_unblock(widget, signal)
+                
+            return retval
+            
+        return wrapper
 #
 # Signal brokers
 #
@@ -139,7 +190,7 @@ class SignalBroker(object):
             if widget is None:
                 raise AttributeError("couldn't find widget %s in %s"
                                      % (w_name, view))
-            if not isinstance(widget, (gtk.Widget, gtk.Action)):
+            if not isinstance(widget, gobject.GObject):
                 raise AttributeError("%s (%s) is not a widget or an action "
                                      "and can't be connected to"
                                      % (w_name, widget))
@@ -415,7 +466,7 @@ class SlaveView(gobject.GObject):
             - widgets: a list of widget names to be searched through
         """
         widget = self.get_topmost_widget(widgets, can_focus=True)
-        if widget is not None: 
+        if widget is not None:
             widget.grab_focus()
         # So it can be idle_added safely
         return False
@@ -449,7 +500,7 @@ class SlaveView(gobject.GObject):
                 if isinstance(widget.get_toplevel(), gtk.Window):
                     widget.realize()
                 else:
-                    _warn("get_topmost_widget: widget %s was unrealized"
+                    _warn("get_topmost_widget: widget %s was not realized"
                           % widget_name)
                     continue
             if can_focus:
@@ -618,9 +669,8 @@ class SlaveView(gobject.GObject):
             win.add_accel_group(group)
 
     def get_slave(self, holder):
-        if self.slaves.has_key(holder):
-            return self.slaves[holder]
-        raise AttributeError, "Unknown slave for holder: %s" % holder
+        return self.slaves.get(holder)
+
 
 
     #
@@ -692,9 +742,13 @@ class SlaveView(gobject.GObject):
                 not isinstance(widget, MixinSupportValidation)):
                 continue
             
-            widget.connect_object('validation-changed',
-                                  self._on_child__validation_changed,
-                                  widget_name)
+            try:
+                widget.connect_object('validation-changed',
+                                      self._on_child__validation_changed,
+                                      widget_name)
+            except TypeError:
+                raise AssertionError("%r does not have a validation-changed "
+                                     "signal." % widget)
 
         proxy = Proxy(self, model, widgets)
         self.proxies.append(proxy)
@@ -733,7 +787,7 @@ class SlaveView(gobject.GObject):
     def force_validation(self):
         self.check_and_notify_validity(force=True)
         
-gobject.type_register(SlaveView)
+type_register(SlaveView)
 
 class BaseView(SlaveView):
     """A view with a toplevel window."""
