@@ -24,19 +24,17 @@
 #            Lorenzo Gil Sanchez <lgs@sicem.biz>
 #
 
-"""Defines an enhanced version of GtkCheckButton"""
+"""GtkCheckButton support for the Kiwi Framework"""
 
-import gobject
 import gtk
 
 from kiwi import ValueUnset
 from kiwi.interfaces import implementsIProxy
 from kiwi.ui.widgets.proxy import WidgetMixin
-from kiwi.utils import gsignal
+from kiwi.utils import gsignal, type_register
 
 class CheckButton(gtk.CheckButton, WidgetMixin):
     implementsIProxy()
-    gsignal('toggled', 'override')
         
     def __init__(self):
         # changed default data_type because checkbuttons can only
@@ -47,11 +45,16 @@ class CheckButton(gtk.CheckButton, WidgetMixin):
         self.show()
     
     def prop_set_data_type(self, data_type):
-        if data_type == bool or data_type is None:
-            WidgetMixin.prop_set_data_type(self, data_type)
-        else:
-            raise TypeError("CheckButtons only accept boolean values")
+        # FIXME: only allow strings or only allow types
+        if (data_type != bool and
+            data_type != 'bool' and
+            data_type is not None):
+            raise TypeError("CheckButtons only accept boolean values and "
+                            "not %r" % data_type)
 
+        WidgetMixin.prop_set_data_type(self, data_type)
+
+    gsignal('toggled', 'override')
     def do_toggled(self):
         self.emit('content-changed')
         self.chain()
@@ -60,9 +63,7 @@ class CheckButton(gtk.CheckButton, WidgetMixin):
         return self.get_active()
 
     def update(self, data):
-        # first, trigger some basic validation
-        WidgetMixin.update(self, data)
         if data is not ValueUnset and data is not None:
             self.set_active(data)
 
-gobject.type_register(CheckButton)
+type_register(CheckButton)
