@@ -22,6 +22,9 @@
 #            Johan Dahlin <jdahlin@async.com.br>
 #
 
+"""GObject utilities and addons"""
+
+import struct
 import sys
 
 import gobject
@@ -41,7 +44,20 @@ def list_properties(gtype, parent=True):
     parent_pspecs = gobject.list_properties(parent)
     return [pspec for pspec in pspecs
                       if pspec not in parent_pspecs]
-            
+
+def type_register(gtype):
+    """Register the type, but only if it's not already registered
+    @param gtype: the class to register
+    """
+
+    # copied from gobjectmodule.c:_wrap_type_register
+    if (getattr(gtype, '__gtype__', None) !=
+        getattr(gtype.__base__, '__gtype__', None)):
+        return False
+
+    gobject.type_register(gtype)
+
+    return True
 
 class PropertyObject(ClassInittableObject):
     """
@@ -83,13 +99,14 @@ class PropertyObject(ClassInittableObject):
 
         # Register the type, here so don't have to do it explicitly, it
         # can be removed in PyGTK 2.8, since it does this magic for us.
-        gobject.type_register(cls)
+        type_register(cls)
 
         # Create python properties for gobject properties, store all
         # the values in self._attributes, so do_set/get_property
         # can access them. Using set property for attribute assignments
         # allows us to add hooks (notify::attribute) when they change.
         default_values = {}
+
         for pspec in list_properties(cls, parent=False):
             prop_name = pspec.name.replace('-', '_')
 
@@ -97,8 +114,13 @@ class PropertyObject(ClassInittableObject):
                          lambda self, v, n=prop_name: self.set_property(n, v))
             setattr(cls, prop_name, p)
 
+            # PyGTK 2.7.1-2.8.0 bugfix
+            if not hasattr(pspec, 'default_value'):
+                default_value = None
+            else:
+                default_value = pspec.default_value
+            
             # Resolve an integer to a real enum
-            default_value = pspec.default_value
             if gobject.type_is_a(pspec.value_type, gobject.GEnum):
                 pyenum = pytypes[prop_name]
                 default_value = pyenum.__enum_values__[default_value]
@@ -115,6 +137,12 @@ class PropertyObject(ClassInittableObject):
             value = func(value)
         self._attributes[name] = value
 
+    def _get(self, name):
+        func = getattr(self, 'prop_get_%s' % name, None)
+        if callable(func) and func:
+            return func()
+        return self._attributes[name]
+
     def get_attribute_names(self):
         return self._attributes.keys()
 
@@ -127,14 +155,25 @@ class PropertyObject(ClassInittableObject):
         
     def do_get_property(self, pspec):
         prop_name = pspec.name.replace('-', '_')
-        return self._attributes[prop_name]
+        return self._get(prop_name)
     
 def gsignal(name, *args, **kwargs):
     """
     Add a GObject signal to the current object.
-    @type name:   string
-    @type args:   types
-    @type kwargs: keyword argument 'flags' and/or 'retval'
+    @param name:     name of the signal
+    @type name:      string
+    @param args:     types for signal parameters,
+      if the first one is a string 'override', the signal will be
+      overridden and must therefor exists in the parent GObject.
+    @keyword flags: One of the following:
+      - gobject.SIGNAL_RUN_FIRST
+      - gobject.SIGNAL_RUN_LAST
+      - gobject.SIGNAL_RUN_CLEANUP
+      - gobject.SIGNAL_NO_RECURSE
+      - gobject.SIGNAL_DETAILED
+      - gobject.SIGNAL_ACTION
+      - gobject.SIGNAL_NO_HOOKS
+    @keyword retval: return value in signal callback
     """
 
     frame = sys._getframe(1)
@@ -156,18 +195,32 @@ def gsignal(name, *args, **kwargs):
     
         dict[name] = (flags, retval, args)
 
-_MAX_INT = sys.maxint
-_MAX_FLOAT = 1e+308
-_MAX_LONG = 18446744073709551616L
+def _max(c):
+   return (1 << (8 * struct.calcsize(c)-1))-1
+
+_MAX_INT = int(_max('i'))
+_MAX_FLOAT = float(_max('f'))
+_MAX_LONG = long(_max('l'))
 
 def gproperty(name, type, *args, **kwargs):
     """
     Add a GObject property to the current object.
-    @type type:    type
-    @type default: default value
+    @param name:   name of property
     @type name:    string
-    @type nick:    string
-    @type flags:   a gobject.ParamFlag
+    @param type:   type of property
+    @type type:    type
+    @keyword default:  default value
+    @keyword nick:     short description
+    @keyword blurb:    long description
+    @keyword minimum:  minimum allowed value (only for int, float, long)
+    @keyword maximum:  maximum allowed value (only for int, float, long)
+    @keyword flags:    parameter flags, one of:
+      - PARAM_READABLE
+      - PARAM_READWRITE
+      - PARAM_WRITABLE
+      - PARAM_CONSTRUCT
+      - PARAM_CONSTRUCT_ONLY
+      - PARAM_LAX_VALIDATION
     """
 
     frame = sys._getframe(1)
@@ -216,25 +269,3 @@ def gproperty(name, type, *args, **kwargs):
 
     dict[name] = tuple(args)
 
-def clamp(x, low, high):
-    """
-    Ensures that x is between the limits set by low and high.
-    For example,
-    * clamp(5, 10, 15) is 10.
-    * clamp(15, 5, 10) is 10.
-    * clamp(20, 15, 25) is 20. 
-
-    @param    x: the value to clamp.
-    @param  low: the minimum value allowed.
-    @param high: the maximum value allowed.
-    """
-    
-    return min(max(x, low), high)
-
-def slicerange(slice, limit):
-    """Takes a slice object and returns a range iterator
-
-    @param slice: slice object
-    @param limit: maximum value allowed"""
-    
-    return xrange(*slice.indices(limit))
